/***************************************************************************
 *  colitas.h : This file is part of 'colitas'
 *
 *  (c) 2004 by Lukasz Tomicki <tomicki@o2.pl>
 *
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef __COLITAS_H_
#define __COLITAS_H_

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <net/if.h> 
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <time.h>
#include <vector>

typedef unsigned int u32;
typedef unsigned short int u16;
typedef unsigned char u8;

class CScanner {
public:
class scan_data {
	public:
	scan_data() { };
	scan_data(scan_data* in) : s1(in->s1), s2(in->s2), s3(in->s3), s4(in->s4),
	e1(in->e1), e2(in->e2), e3(in->e3), e4(in->e4)
	{ };
	u8 s1;
	u8 s2;
	u8 s3;
	u8 s4;
	u8 e1;
	u8 e2;
	u8 e3;
	u8 e4;
};	

private:
static const u16 max_port = 65025;
	
class current_scan_data {
public:
	current_scan_data() { };
	current_scan_data(scan_data& p) : s1(p.s1), s2(p.s2), s3(p.s3), s4(p.s4),
	port(0) { };
	u8 s1;
	u8 s2;
	u8 s3;
	u8 s4;
	u16 port;
};

struct pseudohdr {
  unsigned long saddr;
  unsigned long daddr;
  char useless;
  unsigned char protocol;
  unsigned short length;
};	

public: 
	CScanner();	

	bool start_scan();
	void populate_port_list(const char*);
	void populate_ip_addr(const char*);
	void get_options(int, char**);
	void set_output(void (*p)(char const*));
	u32 get_ports();
	u32 get_hosts();

	scan_data input_data;
	std::vector<u16> ports;
	bool ports_first;
	u32 scan_speed;
	u32 wait_seconds;

protected:
	bool check_input_data(scan_data input);
	void syn_send(unsigned int saddr, unsigned int daddr, unsigned short sport, 
	unsigned short dport);
	void syn_recv(u32, u32 = 0);
	u16 get_next_port();
	u16 in_cksum(unsigned short *ptr, int nbytes);
	unsigned long get_next_host_port();
	unsigned long get_next_port_host();
	unsigned long get_own_ip();
	unsigned long get_next_addr();
	
	void (*output_func)(char const*);
	int sockfd;
	current_scan_data out_data;
	u32 ack_code;
	u32 seq_ack;
};

#endif // __COLITAS_H_
